@@ -1,242 +1,330 @@
 #!/usr/bin/perl -w
 use strict;
-use Text::Wrap;
-$Text::Wrap::columns = 80;
 my ($committer, $patch, $author, $date);
 use Getopt::Long;
+use Text::Wrap;
+$Text::Wrap::columns = 80;
 
-my ($rank, $percentage, $cumulative, $reverse, $ta, @authors, %authors,
-    %untraced, %patchers, %committers, %real_names);
-my $result = GetOptions ("rank" => \$rank,            # rank authors
-             "thanks-applied" => \$ta,        # ranks committers
-             "acknowledged=s"   => \@authors ,  # authors files
+my ($rank, $ta, $ack, $who, $tap) = (0) x 5;
+my ($author_file, $percentage, $cumulative, $reverse);
+my (%authors, %untraced, %patchers, %committers, %real_names);
+
+my $result = GetOptions (
+             # modes
+             "who" => \$who,
+             "rank" => \$rank,
+             "thanks-applied" => \$ta,
+             "missing"   => \$ack ,
+             "tap" => \$tap,
+             # modifiers
+             "authors" => \$author_file,
              "percentage" => \$percentage,      # show as %age
              "cumulative" => \$cumulative,
              "reverse" => \$reverse,
             );
 
-if (!$result or (($rank||0) + ($ta||0) + (@authors ? 1 : 0) != 1) or !@ARGV) {
+if (!$result or ( $rank + $ta + $who + $ack + $tap != 1 ) or !@ARGV) {
+    usage();
+}
+
+$author_file ||= './AUTHORS';
+die "Can't locate '$author_file'. Specify it with '--author <path>'."
+  unless -f $author_file;
+
+my $map = generate_known_author_map();
+
+read_authors_files($author_file);
+
+parse_commits_from_stdin();
+
+if ($rank) {
+  display_ordered(\%patchers);
+} elsif ($ta) {
+  display_ordered(\%committers);
+} elsif ($tap) {
+  display_test_output(\%patchers, \%authors, \%real_names);
+} elsif ($ack) {
+  display_missing_authors(\%patchers, \%authors, \%real_names);
+} elsif ($who) {
+  list_authors(\%patchers, \%authors);
+}
+
+exit(0);
+
+sub usage {
+
   die <<"EOS";
-$0 --rank changes                           # rank authors by patches
-$0 --acknowledged <authors file> changes    # Display unacknowledged authors
-$0 --thanks-applied changes                 # ranks committers of others' patches
-$0 --percentage ...                         # show rankings as percentages
-$0 --cumulative ...                         # show rankings cumulatively
-$0 --reverse ...                            # show rankings in reverse
-Specify stdin as - if needs be. Remember that option names can be abbreviated.
-Generate changes with git log --pretty=fuller rev1..rev2
+Usage: $0 [modes] [modifiers] <git-log-output-file>
+
+Modes (use only one):
+   --who                          # show list of unique authors by full name
+   --rank                         # rank authors by patches
+   --thanks-applied               # ranks committers of others' patches
+   --missing                      # display authors not in AUTHORS
+   --tap                          # show authors present/missing as TAP
+
+Modifiers:
+   --authors <authors-file>       # path to authors file (default: ./AUTHORS)
+   --percentage                   # show rankings as percentages
+   --cumulative                   # show rankings cumulatively
+   --reverse                      # show rankings in reverse
+
+Generate git-log-output-file with git log --pretty=fuller rev1..rev2
+(or pipe by specifing '-' for stdin).  For example:
+  \$ git log --pretty=fuller v5.12.0..v5.12.1 > gitlog
+  \$ perl Porting/checkAUTHORS.pl --rank --percentage gitlog
 EOS
 }
 
-
-my $prev = "";
-my %map;
-
-while (<DATA>) {
-    chomp;
-    s/\\100/\@/g;
-    $_ = lc;
-    if (my ($correct, $alias) = /^\s*([^#\s]\S*)\s+(.*\S)/) {
-        $correct =~ s/^\\043/#/;
-        if ($correct eq '+') {$correct = $prev} else {$prev = $correct}
-        $map {$alias} = $correct;
-    }
+sub list_authors {
+    my ($patchers, $authors) = @_;
+    binmode(STDOUT, ":utf8");
+    print "$_\n" for  sort { lc $a cmp lc $b }
+                      map { $authors->{$_} }
+                      keys %$patchers;
 }
 
-#
-# Email addresses for we do not have names.
-#
-$map {$_} = "?" for 
-    "bah\100longitude.com",
-    "bbucklan\100jpl-devvax.jpl.nasa.gov",
-    "bilbo\100ua.fm",
-    "bob\100starlabs.net",
-    "cygwin\100cygwin.com",
-    "david\100dhaller.de",
-    "erik\100cs.uni-jena.de",
-    "info\100lingo.kiev.ua", # Lingo Translation agency
-    "jms\100mathras.comcast.net",
-    "premchai21\100yahoo.com",
-    "pxm\100nubz.org",
-    "raf\100tradingpost.com.au",
-    "smoketst\100hp46t243.cup.hp.com",
-    "root\100chronos.fi.muni.cz", # no clue - jrv 20090803
-    "gomar\100md.media-web.de", # no clue - jrv 20090803
-    "data-drift\100so.uio.no", # no data. originally private message from 199701282014.VAA12645@selters.uio.no
-    "arbor\100al37al08.telecel.pt", # reported perlbug ticket 5196 - no actual code contribution. no real name - jrv 20091006
-    "oracle\100pcr8.pcr.com", # Reported perlbug ticket 1015 - no patch - Probably Ed Eddington ed@pcr.com
-    ;
+sub parse_commits_from_stdin {
+    my @lines = split( /^commit\s*/sm, join( '', <> ) );
+    for (@lines) {
+        next if m/^$/;
+        next if m/^(\S*?)^Merge:/ism;    # skip merge commits
+        if (m/^(.*?)^Author:\s*(.*?)^AuthorDate:\s*(.*?)^Commit:\s*(.*?)^(.*)$/gism) {
 
-#
-# Email addresses for people that don't have an email address in AUTHORS
-# Presumably deliberately?
-# 
-
-$map {$_} = '!' for
-    # Nick Ing-Simmons has passed away (2006-09-25).
-    "nick\100ing-simmons.net",
-    "nik\100tiuk.ti.com",
-    "nick.ing-simmons\100elixent.com",
-    "nick\100ni-s.u-net.com",
-    "nick.ing-simmons\100tiuk.ti.com",
-
-    # Iain Truskett has passed away (2003-12-29).
-    "perl\100dellah.anu.edu.au",
-    "spoon\100dellah.org",
-    "spoon\100cpan.org",
-
-    # Ton Hospel
-    "me-02\100ton.iguana.be",
-    "perl-5.8.0\100ton.iguana.be",
-    "perl5-porters\100ton.iguana.be",
-
-    # Beau Cox
-    "beau\100beaucox.com",
-
-    # Randy W. Sims 
-    "ml-perl\100thepierianspring.org",
-
-    # perl internal addresses
-    "perl5-porters\100africa.nicoh.com",
-    "perlbug\100perl.org",,
-    "perl5-porters.nicoh.com",
-    "perlbug-followup\100perl.org",
-    "perlbug-comment\100perl.org",
-    "bug-module-corelist\100rt.cpan.org",
-    "bug-storable\100rt.cpan.org",
-    "bugs-perl5\100bugs6.perl.org",
-    "unknown",
-    "unknown\100unknown",
-    "unknown\100longtimeago",
-    "unknown\100perl.org",
-    "",
-    "(none)",
-    ;
-
-
-if (@authors) {
-  my %raw;
-  foreach my $filename (@authors) {
-    open FH, "<$filename" or die "Can't open $filename: $!";
-    while (<FH>) {
-      next if /^\#/;
-      next if /^-- /;
-      if (/<([^>]+)>/) {
-    # Easy line.
-    $raw{$1}++;
-      } elsif (/^([-A-Za-z0-9 .\'--]+)[\t\n]/) {
-    # Name only
-    $untraced{$1}++;
-      } elsif (length $_) {
-    chomp;
-    warn "Can't parse line '$_'";
-      } else {
-	  next
-      }
+            # new patch
+            ( $patch, $author, $date, $committer ) = ( $1, $2, $3, $4 );
+            chomp($author);
+            unless ($author) { die $_ }
+            chomp($committer);
+            unless ($committer) { die $_ }
+            process( $committer, $patch, $author );
+        } else {
+            die "XXX $_ did not match";
+        }
     }
-  }
-  foreach (keys %raw) {
-    print "E-mail $_ occurs $raw{$_} times\n" if $raw{$_} > 1;
-    $_ = lc $_;
-    $authors{$map{$_} || $_}++;
-  }
-  ++$authors{'!'};
-  ++$authors{'?'};
-}
 
-my @lines = split(/^commit\s*/sm,join('',<>));
-for ( @lines) {
-  next if m/^$/;
-  next if m/^(\S*?)^Merge:/ism; # skip merge commits
-if (m/^(.*?)^Author:\s*(.*?)^AuthorDate:\s*(.*?)^Commit:\s*(.*?)^(.*)$/gism) {
-    # new patch
-    ($patch, $author, $date, $committer) = ($1,$2,$3,$4);
-    chomp($author);
-    unless ($author) { die $_}
-    chomp($committer);
-    unless ($committer) { die $_}
-    &process($committer, $patch, $author);
-} else { die "XXX $_ did not match";}
 }
 
 
-if ($rank) {
-  &display_ordered(\%patchers);
-} elsif ($ta) {
-  &display_ordered(\%committers);
-} elsif (%authors) {
-  my %missing;
-  foreach (sort keys %patchers) {
-    next if $authors{$_};
-    # Sort by number of patches, then name.
-    $missing{$patchers{$_}}->{$_}++;
-  }
-  foreach my $patches (sort {$b <=> $a} keys %missing) {
-    print "\n\n=head1 $patches patch(es)\n\n";
-    foreach my $author (sort keys %{$missing{$patches}}) {
-        my $xauthor = $author;
-        $xauthor =~ s/@/\\100/g; # xxx temp hack
-          print "".($real_names{$author}||$author) ."\t\t\t<" . $xauthor.">\n" ;
+sub generate_known_author_map {
+    my %map;
+
+    my $prev = "";
+    while (<DATA>) {
+        chomp;
+        s/\\100/\@/g;
+        $_ = lc;
+        if ( my ( $correct, $alias ) = /^\s*([^#\s]\S*)\s+(.*\S)/ ) {
+            $correct =~ s/^\\043/#/;
+            if   ( $correct eq '+' ) { $correct = $prev }
+            else                     { $prev    = $correct }
+            $map{$alias} = $correct;
+        }
+    }
+
+    #
+    # Email addresses for we do not have names.
+    #
+    $map{$_} = "?"
+        for
+        "bah\100longitude.com",
+        "bbucklan\100jpl-devvax.jpl.nasa.gov",
+        "bilbo\100ua.fm",
+        "bob\100starlabs.net",
+        "cygwin\100cygwin.com",
+        "david\100dhaller.de", "erik\100cs.uni-jena.de", "info\100lingo.kiev.ua",    # Lingo Translation agency
+        "jms\100mathras.comcast.net",
+        "premchai21\100yahoo.com",
+        "pxm\100nubz.org",
+        "raf\100tradingpost.com.au",
+        "smoketst\100hp46t243.cup.hp.com", "root\100chronos.fi.muni.cz",             # no clue - jrv 20090803
+        "gomar\100md.media-web.de",    # no clue - jrv 20090803
+        "data-drift\100so.uio.no",     # no data. originally private message from 199701282014.VAA12645@selters.uio.no
+        "arbor\100al37al08.telecel.pt"
+        ,    # reported perlbug ticket 5196 - no actual code contribution. no real name - jrv 20091006
+        "oracle\100pcr8.pcr.com",    # Reported perlbug ticket 1015 - no patch - Probably Ed Eddington ed@pcr.com
+        ;
+
+    #
+    # Email addresses for people that don't have an email address in AUTHORS
+    # Presumably deliberately?
+    #
+
+    $map{$_} = '!' for
+
+        # Nick Ing-Simmons has passed away (2006-09-25).
+        "nick\100ing-simmons.net",
+        "nik\100tiuk.ti.com",
+        "nick.ing-simmons\100elixent.com",
+        "nick\100ni-s.u-net.com",
+        "nick.ing-simmons\100tiuk.ti.com",
+
+        # Iain Truskett has passed away (2003-12-29).
+        "perl\100dellah.anu.edu.au", "spoon\100dellah.org", "spoon\100cpan.org",
+
+        # Ton Hospel
+        "me-02\100ton.iguana.be", "perl-5.8.0\100ton.iguana.be", "perl5-porters\100ton.iguana.be",
+
+        # Beau Cox
+        "beau\100beaucox.com",
+
+        # Randy W. Sims
+        "ml-perl\100thepierianspring.org",
+
+        # perl internal addresses
+        "perl5-porters\100africa.nicoh.com",
+        "perlbug\100perl.org",,
+        "perl5-porters.nicoh.com",
+        "perlbug-followup\100perl.org",
+        "perlbug-comment\100perl.org",
+        "bug-module-corelist\100rt.cpan.org",
+        "bug-storable\100rt.cpan.org",
+        "bugs-perl5\100bugs6.perl.org",
+        "unknown",
+        "unknown\100unknown",
+        "unknown\100longtimeago",
+        "unknown\100perl.org",
+        "",
+        "(none)",
+        ;
+
+    return \%map;
+}
+
+sub read_authors_files {
+    my @authors = (@_);
+    return unless (@authors);
+    my (%count, %raw);
+    foreach my $filename (@authors) {
+        open FH, "<$filename" or die "Can't open $filename: $!";
+        binmode FH, ':encoding(ISO-8859-1)';
+        while (<FH>) {
+            next if /^\#/;
+            next if /^-- /;
+            if (/^([^<]+)<([^>]+)>/) {
+                # Easy line.
+                my ($name, $email) = ($1, $2);
+                $name =~ s/\s*\z//;
+                $raw{$email} = $name;
+                $count{$email}++;
+            } elsif (/^([-A-Za-z0-9 .\'--]+)[\t\n]/) {
+
+                # Name only
+                $untraced{$1}++;
+            } elsif ( length $_ ) {
+                chomp;
+                warn "Can't parse line '$_'";
+            } else {
+                next;
+            }
+        }
+    }
+    foreach ( keys %raw ) {
+        print "E-mail $_ occurs $count{$_} times\n" if $count{$_} > 1;
+        my $lc = lc $_;
+        $authors{ $map->{$lc} || $lc } = $raw{$_};
+    }
+    $authors{$_} = $_ for qw(? !);
+}
+
+sub display_test_output {
+    my $patchers   = shift;
+    my $authors    = shift;
+    my $real_names = shift;
+    my $count = 0;
+    foreach ( sort keys %$patchers ) {
+           $count++;
+
+        if ($authors->{$_}) {
+            print "ok $count - ".$real_names->{$_} ." $_\n";
+        } else {
+            print "not ok $count - Contributor not found in AUTHORS: $_ ".($real_names->{$_} || '???' )."\n";
+        }
+
+    }
+    print "1..$count\n";
+}
+
+sub display_missing_authors {
+    my $patchers   = shift;
+    my $authors    = shift;
+    my $real_names = shift;
+    my %missing;
+    foreach ( sort keys %$patchers ) {
+        next if $authors->{$_};
+
+        # Sort by number of patches, then name.
+        $missing{ $patchers{$_} }->{$_}++;
+    }
+    foreach my $patches ( sort { $b <=> $a } keys %missing ) {
+        print "\n\n=head1 $patches patch(es)\n\n";
+        foreach my $author ( sort keys %{ $missing{$patches} } ) {
+            my $xauthor = $author;
+            $xauthor =~ s/@/\\100/g;    # xxx temp hack
+            print "" . ( $real_names->{$author} || $author ) . "\t\t\t<" . $xauthor . ">\n";
+        }
     }
-  }
 }
 
 sub display_ordered {
-  my $what = shift;
-  my @sorted;
-  my $total;
-  while (my ($name, $count) = each %$what) {
-    push @{$sorted[$count]}, $name;
-    $total += $count;
-  }
+    my $what = shift;
+    my @sorted;
+    my $total;
 
-  my $i = @sorted;
-  return unless @sorted;
-  my $sum = 0;
-  foreach my $i ($reverse ? 0 .. $#sorted : reverse 0 .. $#sorted) {
-    next unless $sorted[$i];
-    my $prefix;
-    $sum += $i * @{$sorted[$i]};
-    # Value to display is either this one, or the cumulative sum.
-    my $value = $cumulative ? $sum : $i;
-    if ($percentage) {
-    $prefix = sprintf "%6.2f:\t", 100 * $value / $total;
-    } else {
-    $prefix = "$value:\t";
+    while ( my ( $name, $count ) = each %$what ) {
+        push @{ $sorted[$count] }, $name;
+        $total += $count;
+    }
+
+    my $i = @sorted;
+    return unless @sorted;
+    my $sum = 0;
+    foreach my $i ( $reverse ? 0 .. $#sorted : reverse 0 .. $#sorted ) {
+        next unless $sorted[$i];
+        my $prefix;
+        $sum += $i * @{ $sorted[$i] };
+
+        # Value to display is either this one, or the cumulative sum.
+        my $value = $cumulative ? $sum : $i;
+        if ($percentage) {
+            $prefix = sprintf "%6.2f:\t", 100 * $value / $total;
+        } else {
+            $prefix = "$value:\t";
+        }
+        print wrap ( $prefix, "\t", join( " ", sort @{ $sorted[$i] } ), "\n" );
     }
-    print wrap ($prefix, "\t", join (" ", sort @{$sorted[$i]}), "\n");
-  }
 }
 
 sub process {
-  my ($committer, $patch, $author) = @_;
-  return unless $author;
-  return unless $committer;
+    my ( $committer, $patch, $author ) = @_;
+    return unless $author;
+    return unless $committer;
 
-  $author = _raw_address($author);
-  $patchers{$author}++;
+    $author = _raw_address($author);
+    $patchers{$author}++;
 
-  $committer = _raw_address($committer);
-  if ($committer ne $author) {
-    # separate commit credit only if committing someone else's patch
-    $committers{$committer}++;
-  }
+    $committer = _raw_address($committer);
+    if ( $committer ne $author ) {
+
+        # separate commit credit only if committing someone else's patch
+        $committers{$committer}++;
+    }
 }
 
 sub _raw_address {
     my $addr = shift;
     my $real_name;
-    if ($addr =~ /<.*>/) {
-    $addr =~ s/^\s*(.*)\s*<\s*(.*?)\s*>.*$/$2/ ;
-     $real_name = $1;
+    if ( $addr =~ /<.*>/ ) {
+        $addr =~ s/^\s*(.*)\s*<\s*(.*?)\s*>.*$/$2/;
+        $real_name = $1;
     }
     $addr =~ s/\[mailto://;
     $addr =~ s/\]//;
     $addr = lc $addr;
-    $addr = $map{$addr} || $addr;
-    $addr =~ s/\\100/@/g;  # Sometimes, there are encoded @ signs in the git log.
+    $addr = $map->{$addr} || $addr;
+    $addr =~ s/\\100/@/g;    # Sometimes, there are encoded @ signs in the git log.
 
-    if ($real_name) { $real_names{$addr} = $real_name};
+    if ($real_name) { $real_names{$addr} = $real_name }
     return $addr;
 }
 
@@ -247,7 +335,7 @@ __DATA__
 # List of mappings. First entry the "correct" email address, as appears
 # in the AUTHORS file. Second is any "alias" mapped to it.
 #
-# If the "correct" email address is a '+', the entry above is reused; 
+# If the "correct" email address is a '+', the entry above is reused;
 # this for addresses with more than one alias.
 #
 # Note that all entries are in lowercase. Further, no '@' signs should
@@ -326,6 +414,8 @@ merijn                                  h.m.brand\100xs4all.nl
 +                                       merijn\100a5.(none)
 mhx                                     mhx-perl\100gmx.net
 +                                       mhx\100r2d2.(none)
+mst                                     mst\100shadowcat.co.uk
++                                       matthewt\100hercule.scsys.co.uk
 nicholas                                nick\100unfortu.net
 +                                       nick\100ccl4.org
 +                                       nick\100talking.bollo.cx
@@ -333,6 +423,7 @@ nicholas                                nick\100unfortu.net
 +                                       nick\100babyhippo.co.uk
 +                                       nick\100bagpuss.unfortu.net
 +                                       nick\100babyhippo.com
++                                       nicholas\100dromedary.ams6.corp.booking.com
 +                                       Nicholas Clark (sans From field in mail header)
 pudge                                   pudge\100pobox.com
 rgs                                     rgarciasuarez\100free.fr
@@ -352,7 +443,8 @@ stevep                                  steve\100fisharerojo.org
 +                                       root\100dixie.cscaper.com
 timb                                    Tim.Bunce\100pobox.com
 +                                       tim.bunce\100ig.co.uk
-
+tonyc                                   tony\100develop-help.com
++                                       tony\100openbsd32.tony.develop-help.com
 
 #
 # Mere mortals.
@@ -405,6 +497,7 @@ anno4000\100lublin.zrz.tu-berlin.de     anno4000\100mailbox.tu-berlin.de
 +                                       siegel\100zrz.tu-berlin.de
 arnold\100gnu.ai.mit.edu                arnold\100emoryu2.arpa
 +                                       gatech!skeeve!arnold
+arodland\100cpan.org                    andrew\100hbslabs.com
 arussell\100cs.uml.edu                  adam\100adam-pc.(none)
 ash\100cpan.org                         ash_cpan\100firemirror.com
 avarab\100gmail.com                     avar\100cpan.org
@@ -422,6 +515,8 @@ ben\100morrow.me.uk                     mauzo\100csv.warwick.ac.uk
 +                                       mauzo\100.(none)
 bepi\100perl.it                         enrico.sorcinelli\100gmail.com
 bert\100alum.mit.edu                    bert\100genscan.com
+bigbang7\100gmail.com                   ddascalescu+github\100gmail.com
+blgl\100stacken.kth.se                  blgl\100hagernas.com
 brian.d.foy\100gmail.com                bdfoy\100cpan.org
 BQW10602\100nifty.com                   sadahiro\100cpan.org
 
@@ -547,8 +642,9 @@ larry\100wall.org                       lwall\100jpl-devvax.jpl.nasa.gov
 +                                       lwall\100scalpel.netlabs.com
 laszlo.molnar\100eth.ericsson.se        molnarl\100cdata.tvnet.hu
 +                                       ml1050\100freemail.hu
-lewart\100uiuc.edu                      lewart\100vadds.cvm.uiuc.edu    
+lewart\100uiuc.edu                      lewart\100vadds.cvm.uiuc.edu
 +                                       d-lewart\100uiuc.edu
+lkundrak\100v3.sk                      lubo.rintel\100gooddata.com
 lstein\100cshl.org                      lstein\100formaggio.cshl.org
 +                                       lstein\100genome.wi.mit.edu
 lupe\100lupe-christoph.de               lupe\100alanya.m.isar.de
@@ -568,6 +664,7 @@ mgjv\100comdyn.com.au                   mgjv\100tradingpost.com.au
 mlh\100swl.msd.ray.com                  webtools\100uewrhp03.msd.ray.com
 michael.schroeder\100informatik.uni-erlangen.de mls\100suse.de
 mike\100stok.co.uk                      mike\100exegenix.com
+miyagawa\100bulknews.net                    miyagawa\100edge.co.jp
 mjtg\100cam.ac.uk                       mjtg\100cus.cam.ac.uk
 mikedlr\100tardis.ed.ac.uk              mikedlr\100it.com.pl
 moritz\100casella.verplant.org          moritz\100faui2k3.org
@@ -593,6 +690,8 @@ ilya\100math.berkeley.edu               ilya\100math.ohio-state.edu
 +                                       [9]ilya\100math.ohio-state.edu
 ilya\100martynov.org                    ilya\100juil.nonet
 
+joshua.pritikin\100db.com               joshua\100paloalto.com
+
 okamoto\100corp.hp.com                  okamoto\100hpcc123.corp.hp.com
 orwant\100oreilly.com                   orwant\100media.mit.edu
 
@@ -601,6 +700,7 @@ p5-authors\100crystalflame.net          perl\100crystalflame.net
 +                                       coral\100eekeek.org
 +                                       coral\100moonlight.crystalflame.net
 +                                       rs\100oregonnet.com
++                                       rs\100topsy.com
 paul.green\100stratus.com               paul_greenvos\100vos.stratus.com
 +                                       pgreen\100seussnt.stratus.com
 paul.marquess\100btinternet.com         paul_marquess\100yahoo.co.uk
@@ -616,11 +716,12 @@ perl\100greerga.m-l.org                 greerga\100m-l.org
 perl\100profvince.com                   vince\100profvince.com
 perl-rt\100wizbit.be                    p5p\100perl.wizbit.be
 # Maybe we should special case this to get real names out?
-Peter.Dintelmann\100Dresdner-Bank.com   peter.dintelmann\100dresdner-bank.com 
+Peter.Dintelmann\100Dresdner-Bank.com   peter.dintelmann\100dresdner-bank.com
 # NOTE: There is an intentional trailing space in the line above
 pfeifer\100wait.de                      pfeifer\100charly.informatik.uni-dortmund.de
 +                                       upf\100de.uu.net
 rabbit\100rabbit.us                     rabbit+bugs\100rabbit.us
+perl\100aaroncrane.co.uk		arc\100cpan.org
 phil\100perkpartners.com                phil\100finchcomputer.com
 pimlott\100idiomtech.com                andrew\100pimlott.net
 +                                       pimlott\100abel.math.harvard.edu
@@ -653,6 +754,7 @@ rmbarker\100cpan.org                    rmb1\100cise.npl.co.uk
 +                                       robin.barker\100npl.co.uk
 +                                       rmb\100cise.npl.co.uk
 +                                       robin\100spade-ubuntu.(none)
++                                       r.m.barker\100btinternet.com
 robertmay\100cpan.org                   rob\100themayfamily.me.uk
 roberto\100keltia.freenix.fr            roberto\100eurocontrol.fr
 robin\100cpan.org                       robin\100kitsite.com
@@ -714,6 +816,7 @@ thomas.dorner\100start.de               tdorner\100amadeus.net
 tjenness\100cpan.org                    t.jenness\100jach.hawaii.edu
 +                                       timj\100jach.hawaii.edu
 tobez\100tobez.org                      tobez\100plab.ku.dk
+toddr\100cpanel.net                     toddr\100cpan.org
 tom\100compton.nu                       thh\100cyberscience.com
 tom.horsley\100mail.ccur.com            tom.horsley\100ccur.com
 +                                       tom\100amber.ssd.hcsc.com
