@@ -1221,15 +1221,11 @@ for my $syms (@syms) {
 
 # variables
 
-if ($define{'MULTIPLICITY'}) {
-    for my $f ($perlvars_h, $intrpvar_h) {
+if ($define{'MULTIPLICITY'} && $define{PERL_GLOBAL_STRUCT}) {
+    for my $f ($perlvars_h) {
 	my $glob = readvar($f, sub { "Perl_" . $_[1] . $_[2] . "_ptr" });
 	emit_symbols $glob;
     }
-    unless ($define{'USE_ITHREADS'}) {
-	# XXX needed for XS extensions that define PERL_CORE
-	emit_symbol("PL_curinterp");
-    }
     # XXX AIX seems to want the perlvars.h symbols, for some reason
     if ($PLATFORM eq 'aix' or $PLATFORM eq 'os2') {	# OS/2 needs PL_thr_key
 	my $glob = readvar($perlvars_h);
@@ -1241,7 +1237,7 @@ else {
 	my $glob = readvar($perlvars_h);
 	emit_symbols $glob;
     }
-    unless ($define{'MULTIPLICITY'}) {
+    unless ($define{MULTIPLICITY}) {
 	my $glob = readvar($intrpvar_h);
 	emit_symbols $glob;
     }
